/*
 * Ripple Echo
 * by a1kageyama (https://www.openprocessing.org/sketch/578572).
 * altered to work with the Capacitive Touch Skin
 ---
 * @author Sjoerd Legué
 * @email sjoerdlegue@gmail.com
 */
 
// NOTE: Set-up Arduino touch interface before starting this script!

import processing.serial.*;
import processing.sound.*;

// Serial connectivity initialisation;
Serial myPort;
String portData;
Boolean firstContact = false;
Boolean inputReady = false;

// Configurable variables;
final int nb = 50;
final int step = 15;
final float DIST = 120;
final float DISTORTION = 40;
Part[][] parts = new Part[nb][nb];
Boolean mode = false;

// SoundFile configuration and ambient variables;
SoundFile exitSound;
SoundFile ambientSound;
int ambientCount = 0;
int ambientLoopOn = 0;
int ambientLoopWait = 1000;

// Point input variables;
int points[] = {0, 0, 0, 0};
int counter = 0;
final int pointLocation[][] = {{122, 644}, {368, 644}, {613, 644}, {122, 460}, {368, 460}, {613, 460}, {122, 276}, {368, 276}, {613, 276}, {122, 92}, {368, 92}, {613, 92}};
int startBox[] = {225, 225};

// Points config;
final int ageTreshold = 1000;
final double factorTreshold = 0.12;
final float jiggle = 0.0;
final int exitSteps = 2;

void setup(){
  //size(2560, 1440, P2D);
  //size(800, 800, P2D);
  fullScreen();
  
  // Setup serial connection with Arduino and buffer data;
  //printArray(Serial.list());
  String portName = Serial.list()[1];
  myPort = new Serial(this, portName, 9600);
  myPort.bufferUntil('\n');
  
  // Set-up soundFile;
  exitSound = new SoundFile(this, "fwoob-1.wav");
  ambientSound = new SoundFile(this, "ambient-1.mp3");
  
  // Set-up points on the screen;
  stroke(20, 20, 20);
  int dx = (width - nb*step)/2;
  int dy = (height - nb*step)/2;
  for(int i=0; i<nb; i++){
    for(int j=0; j<nb; j++){
      int x = i*step + dx;
      int y = j*step + dy;
            
      // Initialise startBox variable if first point;
      if(i == 0 && j == 0) {
        startBox[0] = x;
        startBox[1] = y;
      }
      
      // Add point to array;
      parts[i][j] = new Part(x, y);
    }
  }
}

// Draw is used for input not related to the Serial connection;
void draw(){
  background(0);
  if(inputReady) {
    stroke(248, 202, 0);
  }
  
  // Play sound when loopcount is valid;
  /* if(ambientCount == ambientLoopOn) {
    ambientSound.play();
    
    // Reset counter and set new goal;
    ambientCount = 0;
    ambientLoopOn = ambientLoopWait + (int) random(200, 800);
  }
  ambientCount++;
  println(ambientCount);*/
  
  PVector m = new PVector(0, 0);
  for(int i=0; i<nb; i++){
    for(int j=0; j<nb; j++){
      parts[i][j].update(m);
    }
  }
  
  // Loop over points in cache;
  for(int i=0; points != null && i < points.length; i = i+4) {
    //println(i);
    
    // Get position from point and update;
    if(points[i+1] > 0 && points[i+2] > 0) {
      
      // Remove point if age is too old;
      if(((millis() - points[i]) > ageTreshold)) {
        
        // Count the steps;
        points[i+3]++;
        
        // Play sound if exit is started;
        if(points[i+3] == 1) {
          exitSound.play();
        }
        
        // Animate it moving out, one step at a time;
        int exitStepX = (((width/2) - points[i+1]) / exitSteps) * points[i+3];
        int exitStepY = (((height/2) - points[i+2]) / exitSteps) * points[i+3];
        
        // Calculate new points;
        int x = points[i+1] + exitStepX;
        int y = points[i+2] + exitStepY;
          
        // Update point;
        PVector p = new PVector(x, y);
        for(int q=0; q<nb; q++){
          for(int r=0; r<nb; r++){
            parts[q][r].update(p);
          }
        }
        
        // If all steps have been taken, remove point;
        if(points[i+3] == exitSteps) {
          points[i] = 0;
          points[i+1] = 0;
          points[i+2] = 0;
          points[i+3] = 0;
          break;
        }
      }
      
      PVector p = new PVector(points[i+1] + (int) random(jiggle), points[i+2] + (int) random(jiggle));
      //PVector p = new PVector(points[i+1], points[i+2]);
      for(int j=0; j<nb; j++){
        for(int k=0; k<nb; k++){
          parts[j][k].update(p);
        }
      }
    }
  }
}

// Looping event that can read Serial data;
void serialEvent(Serial myPort) {
  portData = myPort.readStringUntil('\n');
  
  // Check if portData has been sent;
  if (portData != null) {
    portData = trim(portData);
    println(portData);
  
    // Look for handshake bytes;
    if (firstContact == false) {
      if (portData.equals("A")) {
        myPort.clear();
        firstContact = true;
        myPort.write("A");
        println("PSR");
      }
    }
    if (inputReady == false) {
      if (portData.equals("CR")) {
        inputReady = true;
      }
    }
    
    // If handshake has already been performed, just listen;
    else {
      
      // Detect specified commands;
      switch(portData) {
        
        // 'RecordNew'-command;
        case "RN":
          
          break;
          
        // 'RecordDone'-command;
        case "RD":
          
          break;
        
        // Detect 'direction-value-multiplication'-commands;
        default:
          if (Character.isDigit(portData.charAt(0))) {
            
            // Parse incoming data into an integer array;
            String temp[] = portData.split(":");
            int zone = Integer.parseInt(temp[0]);
            double mult = Double.parseDouble(temp[1]);
            
            // Get location from point configuration;
            int x = pointLocation[zone][0] + startBox[0];
            int y = pointLocation[zone][1] + startBox[1];
            
            // Add point to cache if touch factor is higher than treshold;
            if(mult > factorTreshold) {
              points = append(points, millis());
              points = append(points, x);
              points = append(points, y);
              points = append(points, 0); // Exit step counter;
            }
          }
          break;
      }
    }
  }
}

class Part{
  PVector pos, speed, origin;
  Part(int x, int y){
    pos = new PVector(x, y);
    origin = pos.get();
    speed = new PVector(0, 0);
  }
  
  void update(PVector m){
    PVector tmp = origin.get();
    tmp.sub(m);
    float d = tmp.mag();
    float c = map(d, 0, DIST, 0, PI);
    tmp.normalize();
    if(mode){
      tmp.mult(DISTORTION*sin(c));
    }
    if(d<DIST){
      //strokeWeight(1 + 10*abs(cos(c/2)));
      if(!mode){
        tmp.mult(DISTORTION*sin(c));
      }
    }
    else{
      //strokeWeight(map(min(d, width), 0, width, 6, .3));
    }
    strokeWeight(3);
    
    PVector target = new PVector(origin.x + tmp.x, origin.y + tmp.y);
    tmp = pos.get();
    tmp.sub(target);
    tmp.mult(-map(m.dist(pos), 0, 2*width, .1, .01));
    speed.add(tmp);
    speed.mult(.57);
    pos.add(speed);
    
    point(pos.x, pos.y);
  }
}

void mousePressed(){
  for(int i=0; points != null && i < points.length; i++) {
    points[i] = 0;
  }
}
